package pay.platform.api.system.controller;

import cn.hutool.core.bean.BeanUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import pay.platform.api.system.model.query.ConfigQuery;
import pay.platform.api.system.model.vo.ConfigVO;
import pay.platform.api.system.servcie.SysConfigService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.SysConfig;

import java.util.List;

/**
 * 系统配置
 */
@RestController
@RequestMapping("/sys/config")
@Tag(name = "系统：系统配置")
@RequiredArgsConstructor
@Validated
public class SysConfigController {

    private final SysConfigService configService;

    @GetMapping("/query/page")
    @PreAuthorize("hasAuthority('sys:config:query')")
    @Operation(summary = "分页查询")
    public Result<BasePage<ConfigVO>> queryPage(HttpServletRequest request, @ParameterObject ConfigQuery query) {
        return configService.queryPage(query);
    }

    @GetMapping("/query/list")
    @Operation(summary = "列表查询")
    public Result<List<ConfigVO>> queryList(HttpServletRequest request, @ParameterObject ConfigQuery query) {
        return configService.queryList(query);
    }

    @PostMapping("/save")
    @Operation(summary = "保存")
    @PreAuthorize("hasAuthority('sys:config:save')")
    public Result<ConfigVO> save(HttpServletRequest request, @RequestBody ConfigVO vo) {
        String username = SecurityUtil.getCurrentUsername();
        SysConfig config = BeanUtil.toBean(vo, SysConfig.class);
        config.setCreateUser(username);
        config.setUpdateUser(username);
        configService.save(config);
        ConfigVO bean = BeanUtil.toBean(config, ConfigVO.class);
        return Result.OK(bean);
    }

    @PutMapping("/update/{id}")
    @Operation(summary = "修改")
    @PreAuthorize("hasAuthority('sys:config:update')")
    public Result<ConfigVO> updateById(HttpServletRequest request, @PathVariable String id, @RequestBody ConfigVO vo) {
        String username = SecurityUtil.getCurrentUsername();
        SysConfig config = configService.getById(id);
        if (null == config) {
            return Result.NG("数据不存在");
        }
        BeanUtil.copyProperties(vo, config);
        config.setUpdateUser(username);
        configService.updateById(config);
        ConfigVO bean = BeanUtil.toBean(config, ConfigVO.class);
        return Result.OK(bean);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除")
    @PreAuthorize("hasAuthority('sys:config:del')")
    public Result<String> delBatch(HttpServletRequest request, @RequestBody List<String> ids) {
        configService.removeByIds(ids);
        return Result.OK();
    }

    @PutMapping("/refresh/{id}")
    @Operation(summary = "刷新缓存")
    @PreAuthorize("hasAuthority('sys:config:refresh')")
    public Result<String> refreshCache(HttpServletRequest request, @PathVariable String id) {
        return configService.refreshCache(id);
    }
}
