package pay.platform.api.system.controller;

import cn.hutool.core.util.StrUtil;
import pay.platform.api.system.cache.RouteCache;
import pay.platform.api.system.model.dto.PermissionTree;
import pay.platform.api.system.model.query.PermissionTreeQuery;
import pay.platform.api.system.model.vo.PermissionVO;
import pay.platform.api.system.model.vo.VueMenuRouteVO;
import pay.platform.api.system.servcie.SysPermissionService;
import pay.platform.core.common.Result;
import pay.platform.core.enums.MenuTypeEnums;
import pay.platform.core.enums.ValueEnum;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.SysPermission;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * 系统：菜单与权限
 */
@RestController
@RequestMapping("/sys/permission")
@Tag(name = "系统：菜单与权限")
@RequiredArgsConstructor
public class SysPermissionController {

    private final SysPermissionService sysPermissionService;

    private final RouteCache routeCache;

    /**
     * 获取当前用户路由
     *
     * @param request .
     * @return .
     */
    @GetMapping("/routes")
    @Operation(summary = "获取当前用户路由")
    public Result<List<VueMenuRouteVO>> getCurrentRoute(HttpServletRequest request) {
        UserInfo user = SecurityUtil.getCurrentUser();
        Optional<List<VueMenuRouteVO>> routeCacheOptional = routeCache.getCache(user.getUserid());
        if (routeCacheOptional.isPresent()) {
            return Result.OK(routeCacheOptional.get());
        }
        List<VueMenuRouteVO> menuRoute = null;
        if (user.isManger()) {
            Set<String> permissionIds = sysPermissionService.allPermissionIds();
            menuRoute = sysPermissionService.queryRouteByIds(List.copyOf(permissionIds));
        } else {
            menuRoute = sysPermissionService.queryRouteByUserid(user.getUserid());
        }
        routeCache.putCache(user.getUserid(), menuRoute);
        return Result.OK(menuRoute);
    }

    /**
     * 获取菜单与权限
     *
     * @return .
     */
    @GetMapping("/tree")
    @Operation(summary = "获取菜单与权限树形列表")
    @PreAuthorize("@permission.hashPermission('menu:query','role:query')")
    public Result<List<PermissionTree>> treesPermission(@ParameterObject PermissionTreeQuery query) {
        Optional<List<SysPermission>> permissionOpt = sysPermissionService.listPermission(query);
        Optional<List<PermissionTree>> treeOpt = permissionOpt
                .flatMap(e -> Optional.ofNullable(sysPermissionService.buildTree(e)));
        List<PermissionTree> tree = treeOpt.orElse(Collections.emptyList());
        return Result.OK(tree);
    }

    @GetMapping("/tree/menu")
    @Operation(summary = "获取菜单树形列表")
    public Result<List<PermissionTree>> tressMenus(@ParameterObject PermissionTreeQuery query) {
        Optional<List<SysPermission>> permissionOpt = sysPermissionService.listMenu(query);
        Optional<List<PermissionTree>> treeOpt = permissionOpt
                .flatMap(e -> Optional.ofNullable(sysPermissionService.buildTree(e)));
        List<PermissionTree> tree = treeOpt.orElse(Collections.emptyList());
        return Result.OK(tree);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除菜单与权限")
    @PreAuthorize("@permission.hashPermission('menu:del')")
    public Result<String> delete(@Valid @RequestBody List<String> ids) {
        return sysPermissionService.deleteByIds(ids);
    }

    @PostMapping("/save")
    @Operation(summary = "保存菜单与权限")
    @PreAuthorize("@permission.hashPermission('menu:save')")
    public Result<PermissionTree> save(@RequestBody PermissionVO vo) {
        MenuTypeEnums menuType = ValueEnum.valueToEnum(MenuTypeEnums.class, vo.getMenuType());
        Result<PermissionTree> res = null;
        if (MenuTypeEnums.dir == menuType) {
            res = checkDirMenuType(vo);
        } else if (MenuTypeEnums.menu == menuType) {
            res = checkMenuType(vo);
        } else {
            res = checkPermissionType(vo);
        }
        if (null != res) {
            return res;
        }
        String username = SecurityUtil.getCurrentUsername();
        vo.setCreateTime(LocalDateTime.now());
        vo.setCreateUser(username);
        vo.setUpdateTime(LocalDateTime.now());
        vo.setUpdateUser(username);
        Optional<PermissionTree> optional = sysPermissionService.save(vo);
        return optional.map(Result::OK).orElseGet(() -> Result.NG("保存失败"));
    }

    @PutMapping("/update")
    @Operation(summary = "更新菜单与权限")
    @PreAuthorize("@permission.hashPermission('menu:update')")
    public Result<PermissionTree> update(@Parameter(name = "id", description = "当前ID", required = true) @RequestParam("id") String id,
                                         @Valid @RequestBody PermissionVO vo) {
        MenuTypeEnums menuType = ValueEnum.valueToEnum(MenuTypeEnums.class, vo.getMenuType());
        Result<PermissionTree> res = null;
        if (MenuTypeEnums.dir == menuType) {
            res = checkDirMenuType(vo);
        } else if (MenuTypeEnums.menu == menuType) {
            res = checkMenuType(vo);
        } else {
            res = checkPermissionType(vo);
        }
        if (null != res) {
            return res;
        }
        String username = SecurityUtil.getCurrentUsername();
        vo.setUpdateTime(LocalDateTime.now());
        vo.setUpdateUser(username);
        Optional<PermissionTree> optional = sysPermissionService.updateById(vo, id
        );
        return optional.map(Result::OK).orElseGet(() -> Result.NG("更新失败"));
    }

    private Result<PermissionTree> checkDirMenuType(PermissionVO vo) {
        String title = vo.getTitle();
        if (StrUtil.isBlank(title)) {
            return Result.NG("菜单名称为空");
        }
        String path = vo.getPath();
        if (StrUtil.isBlank(path)) {
            return Result.NG("路由地址为空");
        }
        Integer rank = vo.getRank();
        if (null == rank) {
            return Result.NG("显示顺序为空");
        }
        return null;
    }

    private Result<PermissionTree> checkMenuType(PermissionVO vo) {
        String title = vo.getTitle();
        if (StrUtil.isBlank(title)) {
            return Result.NG("菜单名称为空");
        }
        String path = vo.getPath();
        if (StrUtil.isBlank(path)) {
            return Result.NG("路由地址为空");
        }
        Integer rank = vo.getRank();
        if (null == rank) {
            return Result.NG("显示顺序为空");
        }
        return null;
    }

    private Result<PermissionTree> checkPermissionType(PermissionVO vo) {
        String title = vo.getTitle();
        if (StrUtil.isBlank(title)) {
            return Result.NG("权限名称为空");
        }
        String perms = vo.getPerms();
        if (StrUtil.isBlank(perms)) {
            return Result.NG("权限标识为空");
        }
        return null;
    }
}
