package pay.platform.api.system.controller;

import cn.bzvs.otp.OtpAuthUtil;
import cn.hutool.core.util.StrUtil;
import pay.platform.api.system.model.query.UserQuery;
import pay.platform.api.system.model.vo.RestPasswdVO;
import pay.platform.api.system.model.vo.UserVO;
import pay.platform.api.system.servcie.SysUserService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.util.SecurityUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Set;

/**
 * 用户管理
 */
@RestController
@RequestMapping("/sys/user")
@Tag(name = "系统：用户管理")
@RequiredArgsConstructor
public class SysUserController {

    private final SysUserService sysUserService;

    @GetMapping("/query/page")
    @Operation(summary = "分页查询")
    @PreAuthorize("@permission.hashPermission('user:query:page')")
    public Result<BasePage<UserVO>> queryPage(HttpServletRequest request, @ParameterObject UserQuery query) {
        BasePage<UserVO> res = sysUserService.queryPage(query);
        return Result.OK(res);
    }

    @GetMapping("/query/list")
    @Operation(summary = "列表查询")
    public Result<List<UserVO>> queryList(HttpServletRequest request, @ParameterObject UserQuery query) {
        List<UserVO> res = sysUserService.queryList(query);
        return Result.OK(res);
    }

    @GetMapping("/check/username")
    @Operation(summary = "检查用户名是否存在")
    public Result<Boolean> checkUsername(HttpServletRequest request, @RequestParam String username) {
        return sysUserService.hashUsername(username);
    }

    @GetMapping("/")
    @Operation(summary = "详情")
    public Result<UserVO> detail(HttpServletRequest request, @Parameter(name = "id", description = "用户ID",
            required = true) @RequestParam String id) {
        return sysUserService.detail(id);
    }

    @GetMapping("/username")
    @Operation(summary = "详情")
    public Result<UserVO> detailByUsername(HttpServletRequest request, @Parameter(name = "username", description = "用户帐号",
            required = true) @RequestParam String username) {
        return sysUserService.detailByUsername(username);
    }

    @GetMapping("/query/role/ids")
    @Operation(summary = "查询用户角色")
    public Result<Set<String>> queryRoleIds(HttpServletRequest request,
                                            @Parameter(name = "id", description = "用户ID", required = true) @RequestParam String id) {
        return sysUserService.getRoleIdsByUserId(id);
    }

    @PostMapping("/save")
    @Operation(summary = "保存")
    @PreAuthorize("@permission.hashPermission('user:save')")
    public Result<UserVO> save(HttpServletRequest request, @Validated @RequestBody UserVO userVO) {
        userVO.setCreateTime(LocalDateTime.now());
        userVO.setCreateUser(SecurityUtil.getCurrentUsername());
        userVO.setUpdateTime(LocalDateTime.now());
        userVO.setUpdateUser(SecurityUtil.getCurrentUsername());
        return sysUserService.save(userVO);
    }


    @PutMapping("/update/{id}")
    @Operation(summary = "修改")
    @PreAuthorize("@permission.hashPermission('user:update')")
    public Result<UserVO> update(HttpServletRequest request, @PathVariable String id,
                                 @Validated @RequestBody UserVO userVO) {
        userVO.setPassword(null);
        userVO.setUsername(null);
        userVO.setUpdateTime(LocalDateTime.now());
        userVO.setUpdateUser(SecurityUtil.getCurrentUsername());
        return sysUserService.updateById(id, userVO);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除")
    @PreAuthorize("@permission.hashPermission('user:del')")
    public Result<String> delete(HttpServletRequest request, @Validated @RequestBody List<String> ids) {
        if (null == ids || ids.isEmpty()) {
            return Result.NG("删除失败，缺少参数");
        }
        return sysUserService.deleteByIds(ids);
    }

    @PutMapping("/reset/passwd/{username}")
    @Operation(summary = "重置密码")
    @PreAuthorize("@permission.hashPermission('user:reset:passwd')")
    public Result<String> resetPasswd(HttpServletRequest request, @PathVariable(value = "username") String username,
                                      @Validated @RequestBody RestPasswdVO vo) {
        if (StrUtil.isBlank(vo.getNewPassword())) {
            return Result.NG("新密码不能为空");
        }
        if (!vo.getNewPassword().equals(vo.getConfirmPassword())) {
            return Result.NG("两次密码不一致");
        }
        return sysUserService.resetPassword(username, vo.getConfirmPassword());
    }

    @PutMapping("/change/passwd/{username}")
    @Operation(summary = "修改密码")
    public Result<String> changePasswd(HttpServletRequest request,
                                       @PathVariable("username") String username, @Validated @RequestBody RestPasswdVO vo) {
        if (StrUtil.isBlank(vo.getNewPassword())) {
            return Result.NG("新密码不能为空");
        }
        if (!vo.getNewPassword().equals(vo.getConfirmPassword())) {
            return Result.NG("两次密码不一致");
        }
        if (vo.getOldPassword().equals(vo.getNewPassword())) {
            return Result.NG("新密码不能与旧密码相同");
        }
        return sysUserService.changePasswd(username, vo.getOldPassword(), vo.getNewPassword());
    }

    @GetMapping("/get/google/qrcode")
    @Operation(summary = "获取谷歌二维码")
    public Result<String> getGoogleQrcode() {
        UserInfo userInfo = SecurityUtil.getCurrentUser();
        String otp = sysUserService.createGoogleOtp(userInfo.getUserid());
        OtpAuthUtil.getOtpAuthUrl("Game-System", otp);
        return Result.OK(otp);
    }


    @PostMapping("/check/google/otp")
    @Operation(summary = "校验谷歌验证码")
    public Result<Boolean> googleOtpCheck(String code) {
        UserInfo userInfo = SecurityUtil.getCurrentUser();
        boolean status = sysUserService.checkGoogleOtp(userInfo.getUserid(), code);
        return Result.OK(status);
    }
}
