package pay.platform.api.system.servcie;

import cn.bzvs.otp.OtpAuthUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.StringUtil;
import jakarta.annotation.Nonnull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import pay.platform.api.system.model.query.UserQuery;
import pay.platform.api.system.model.vo.UserVO;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.util.PasswordUtil;
import pay.platform.domain.SysUser;
import pay.platform.domain.table.SysUserTableDef;
import pay.platform.mapper.SysRoleMapper;
import pay.platform.mapper.SysUserMapper;
import pay.platform.mapper.SysUserRoleMapper;

import java.util.*;

/**
 * 系统 - 用户 服务层实现。
 *
 * @author ADMIN
 * @since 1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysUserService extends BaseService<SysUserMapper, SysUser> {

    private final SysRoleMapper sysRoleMapper;

    private final SysUserRoleMapper sysUserRoleMapper;

    /**
     * 根据用户名获取用户信息
     *
     * @param username 用户名
     * @return .
     */
    public Optional<SysUser> loadUserByUsername(String username) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysUserTableDef.SYS_USER).where(SysUserTableDef.SYS_USER.USERNAME.eq(username));
        SysUser sysUser = mapper.selectOneByQuery(queryWrapper);
        return Optional.ofNullable(sysUser);
    }

    /**
     * 获取用户角色编码
     *
     * @param username 用户名称
     * @return 角色编码
     */
    @Nonnull
    public Set<String> getRoleCodeByUsername(String username) {
        Set<String> res = mapper.getRoleCodeByUsername(username);
        return res == null ? Collections.emptySet() : res;
    }

    /**
     * 获取全部的角色编码
     *
     * @return .
     */
    public Set<String> allRoleCode() {
        Set<String> res = sysRoleMapper.allRoleCode();
        return res == null ? Collections.emptySet() : res;
    }


    /**
     * 根据用户ID 获取对应的角色ID集合
     *
     * @param userId 角色ID
     * @return 角色ID集合
     */
    public Set<String> queryRoleIdsByUserId(String userId) {
        Set<String> res = mapper.queryUserRole(userId);
        return res == null ? Collections.emptySet() : res;
    }
    /*===================================================*/

    /**
     * 分页查询
     *
     * @param query .
     * @return .
     */
    public BasePage<UserVO> queryPage(UserQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysUserTableDef.SYS_USER).where("1 = 1").and(SysUserTableDef.SYS_USER.USERNAME.like(query.getUsername(), StringUtil.isNotBlank(query.getUsername()))).and(SysUserTableDef.SYS_USER.NICKNAME.eq(query.getNickname(), StringUtil.isNotBlank(query.getNickname()))).and(SysUserTableDef.SYS_USER.ENABLE.eq(query.getEnable(), query.getEnable() != null)).orderBy(SysUserTableDef.SYS_USER.UPDATE_TIME, false);
        Page<SysUser> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<UserVO> vos = BeanUtil.copyToList(page.getRecords(), UserVO.class);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
    }

    /**
     * 列表查询
     *
     * @param query .
     * @return .
     */
    public List<UserVO> queryList(UserQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysUserTableDef.SYS_USER).where("1 = 1").and(SysUserTableDef.SYS_USER.USERNAME.like(query.getUsername(), StringUtil.isNotBlank(query.getUsername()))).and(SysUserTableDef.SYS_USER.NICKNAME.eq(query.getNickname(), StringUtil.isNotBlank(query.getNickname()))).and(SysUserTableDef.SYS_USER.ENABLE.eq(query.getEnable(), query.getEnable() != null));
        List<SysUser> list = mapper.selectListByQuery(queryWrapper);
        return BeanUtil.copyToList(list, UserVO.class);
    }

    /**
     * 帐号是否存在
     *
     * @param username .
     * @return .
     */
    public Result<Boolean> hashUsername(String username) {
        return Result.OK(loadUserByUsername(username).isPresent());
    }

    /**
     * 根据ID获取详情
     *
     * @param id .
     * @return .
     */
    public Result<UserVO> detail(String id) {
        SysUser user = getById(id);
        UserVO res = BeanUtil.toBean(user, UserVO.class);
        res.setPassword(null);
        return Result.OK(res);
    }

    /**
     * 根据用户名获取详情
     *
     * @param username .
     * @return .
     */
    public Result<UserVO> detailByUsername(String username) {
        Optional<SysUser> user = loadUserByUsername(username);
        if (user.isPresent()) {
            UserVO res = BeanUtil.toBean(user, UserVO.class);
            res.setPassword(null);
            return Result.OK(res);
        } else {
            return Result.NG("用户不存在");
        }
    }

    /**
     * 根据用户ID获取角色ID集合
     *
     * @param id .
     * @return .
     */

    public Result<Set<String>> getRoleIdsByUserId(String id) {
        Set<String> roleIds = queryRoleIdsByUserId(id);
        return Result.OK(roleIds);
    }

    /**
     * 保存用户
     *
     * @param vo 用户信息
     * @return .
     */
    @Transactional(rollbackFor = Exception.class)
    public Result<UserVO> save(UserVO vo) {
        String username = vo.getUsername();
        if (hashUsername(username).getResult()) {
            return Result.NG("帐号已存在");
        }
        String password = vo.getPassword();
        if (StrUtil.isBlank(password)) {
            return Result.NG("密码不能为空");
        }
        String newPassword = PasswordUtil.encoder(vo.getPassword());
        SysUser user = BeanUtil.toBean(vo, SysUser.class);
        user.setPassword(newPassword);
        save(user);
        List<String> roleIds = vo.getRoleIds();
        if (CollectionUtil.isNotEmpty(roleIds)) {
            sysUserRoleMapper.saveUserRole(user.getId(), new HashSet<>(roleIds));
        }
        return Result.OK(vo);
    }

    /**
     * 更新用户
     *
     * @param id .
     * @param vo .
     * @return .
     */
    public Result<UserVO> updateById(String id, UserVO vo) {
        SysUser user = getById(id);
        if (null == user) {
            return Result.NG("用户不存在");
        }
        BeanUtil.copyProperties(vo, user);
        updateById(user);
        List<String> roleIds = vo.getRoleIds();
        sysUserRoleMapper.delUserRoleByUserid(user.getId());
        if (CollectionUtil.isNotEmpty(roleIds)) {
            sysUserRoleMapper.saveUserRole(user.getId(), new HashSet<>(roleIds));
        }
        return Result.OK(vo);
    }

    /**
     * 删除用户
     *
     * @param ids 用户集合
     * @return .
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<String> deleteByIds(List<String> ids) {
        if (CollectionUtil.isEmpty(ids)) {
            return Result.NG("参数不能为空");
        }
        removeByIds(ids);
        sysUserRoleMapper.delUserRoleByUserids(ids);
        return Result.OK();
    }

    /**
     * 重置密码
     *
     * @param username .
     * @param passwd   .
     * @return .
     */

    public Result<String> resetPassword(@Nonnull String username, @Nonnull String passwd) {
        String newPassword = PasswordUtil.encoder(passwd);
        mapper.resetPassword(username, newPassword);
        return Result.OK();
    }

    /**
     * 修改密码
     *
     * @param username  .
     * @param oldPasswd .
     * @param newPasswd .
     * @return .
     */
    public Result<String> changePasswd(@Nonnull String username, @Nonnull String oldPasswd, @Nonnull String newPasswd) {
        Optional<SysUser> sysUser = loadUserByUsername(username);
        if (sysUser.isPresent()) {
            return Result.NG("用户不存在");
        }
        SysUser user = sysUser.get();
        if (!PasswordUtil.matches(oldPasswd, user.getPassword())) {
            return Result.NG("原密码错误");
        }
        String newPassword = PasswordUtil.encoder(newPasswd);
        user.setPassword(newPassword);
        mapper.update(user);
        return Result.OK();
    }

    /**
     * 是否有谷歌验证器
     *
     * @param id ID
     * @return true/false
     */
    public boolean hasGoogleOtp(String id) {
        SysUser sysUser = getById(id);
        return StringUtils.hasText(sysUser.getOtp());
    }

    /**
     * 创建谷歌验证器
     *
     * @param id ID
     * @return otp
     */
    public String createGoogleOtp(String id) {
        SysUser sysUser = getById(id);
        String secret = OtpAuthUtil.createSecret();
        sysUser.setOtp(secret);
        updateById(sysUser);
        return OtpAuthUtil.getOtpQrCodeUrl("", secret);
    }

    /**
     * 校验谷歌验证器
     *
     * @param id   ID
     * @param code 验证码
     * @return true/false
     */
    public boolean checkGoogleOtp(String id, String code) {
        SysUser sysUser = getById(id);
        String secret = sysUser.getOtp();
        if (Boolean.FALSE.equals(StringUtils.hasText(secret))) {
            log.warn("用户未设置谷歌验证器");
            return false;
        }
        return OtpAuthUtil.validateCode(secret, code);
    }
}
