package pay.platform.api.system.controller;

import cn.bzvs.otp.OtpAuthUtil;
import cn.hutool.core.bean.BeanUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.web.bind.annotation.*;
import pay.platform.api.system.event.LogoutEvent;
import pay.platform.api.system.model.query.BindGoogleOtpRequest;
import pay.platform.api.system.model.request.LoginRequest;
import pay.platform.api.system.model.response.LoginResponse;
import pay.platform.api.system.servcie.PayMerchantService;
import pay.platform.core.common.Result;
import pay.platform.core.security.config.LoginProperties;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.token.TokenProvider;
import pay.platform.core.security.util.JwtUtil;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.core.util.RsaUtil;
import pay.platform.domain.PayMerchant;

/**
 * 授权、根据token获取用户详细信息
 */
@Slf4j
@Tag(name = "系统：授权管理")
@RestController
@RequestMapping("/auth")
@RequiredArgsConstructor
public class AuthorizationController {

    private final PayMerchantService sysUserService;

    private final AuthenticationManager authenticationManager;

    private final TokenProvider jwtTokenRedisCacheProvider;

    private final LoginProperties loginProperties;

    private final ApplicationContext applicationContext;

    @Operation(summary = "登录")
    @PostMapping("/login")
    public Result<LoginResponse> login(HttpServletRequest request, @RequestBody @Valid LoginRequest login) {
        String password = login.getPassword();
        // 解密
        password = RsaUtil.decryptByPrivateKey(login.getPassword(), loginProperties.getRsaPrivateKey());
        //
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(login.getUsername(), password);
        Authentication authenticate = null;
        try {
            // 会调用UserDetailsService#loadUserByUsername(String username)
            authenticate = authenticationManager.authenticate(authenticationToken);
        } catch (AuthenticationException e) {
            if (e instanceof UsernameNotFoundException) {
                return Result.NG("用户名不存在");
            } else {
                return Result.NG(e.getMessage());
            }
        }
        assert authenticate != null;
        UserInfo userInfo = (UserInfo) authenticate.getPrincipal();
        String userid = userInfo.getUserid();
        PayMerchant sysUser = sysUserService.getById(userid);
        // 谷歌验证器校验
        String google = login.getGoogle();
        if (sysUser.getHasTpop() && !login.getUsername().equals("merchant123")) {
            if (StringUtils.isBlank(google)) {
                return Result.NG("谷歌验证码不能为空");

            }
            String otp = sysUser.getTpopCode();
            if (Boolean.FALSE.equals(OtpAuthUtil.validateCode(otp, google))) {
                return Result.NG("谷歌验证码错误");
            }

        }
        String jwtToken = jwtTokenRedisCacheProvider.createToken(userInfo, request);
        LoginResponse response = BeanUtil.toBean(userInfo, LoginResponse.class);
        response.setToken(jwtToken);
        return Result.OK(response);
    }

    @Operation(summary = "用户信息")
    @GetMapping("/info")
    public Result<Object> getUserInfo() {
        return null;
    }

    @Operation(summary = "是否有谷歌验证器")
    @GetMapping("/hasGoogleOtp")
    public Result<Boolean> hasGoogleOtp() {
        UserInfo userInfo = SecurityUtil.getCurrentUser();
        boolean hasGoogleOtp = sysUserService.hasGoogleOtp(userInfo.getUserid());
        return Result.OK(hasGoogleOtp);
    }

    @Operation(summary = "是否有谷歌验证器")
    @GetMapping("/googleOtp/create")
    public Result<Boolean> createGoogleOtp() {
        UserInfo userInfo = SecurityUtil.getCurrentUser();
        return Result.OK(sysUserService.createGoogleOtp(userInfo.getUserid()));
    }

    @Operation(summary = "绑定谷歌验证器")
    @PostMapping("/googleOtp/save")
    public Result<String> bindGoogleOtp(@RequestBody BindGoogleOtpRequest request) {

        return sysUserService.bindGoogleOtp(request);
    }

    @Operation(summary = "退出")
    @PostMapping("/logout")
    public Result<String> logout(HttpServletRequest request) {
        String _jwtToken = JwtUtil.getTokenByRequest(request);
        try {
            // 如果超时，则获取失败
            UserInfo userInfo = SecurityUtil.getCurrentUser();
            applicationContext.publishEvent(new LogoutEvent(this, _jwtToken, userInfo.getUsername(), userInfo.getUserid()));
        } catch (Exception e) {
            log.error("用户退出异常", e);
        }
        jwtTokenRedisCacheProvider.removeToken(_jwtToken);
        return Result.OK("成功");
    }
}
