package pay.platform.api.system.servcie.impl;


import cn.bzvs.otp.OtpAuthUtil;
import cn.hutool.core.bean.BeanUtil;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Nonnull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import pay.platform.api.system.model.query.BindGoogleOtpRequest;
import pay.platform.api.system.model.vo.PayAgentVO;
import pay.platform.api.system.model.vo.PayMerchantVO;
import pay.platform.api.system.servcie.PayMerchantService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.core.util.PasswordUtil;
import pay.platform.domain.PayMerchant;
import pay.platform.domain.table.PayMerchantTableDef;
import pay.platform.mapper.PayMerchantMapper;

import java.util.Optional;

/**
 * 商户管理 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
@Slf4j
public class PayMerchantServiceImpl extends BaseService<PayMerchantMapper, PayMerchant> implements PayMerchantService {

    /**
     * 根据用户名获取用户信息
     *
     * @param username 用户名
     * @return .
     */
    public Optional<PayMerchant> loadUserByUsername(String username) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayMerchantTableDef.PAY_MERCHANT).where(PayMerchantTableDef.PAY_MERCHANT.USER_NAME.eq(username));
        PayMerchant sysUser = mapper.selectOneByQuery(queryWrapper);
        return Optional.ofNullable(sysUser);
    }

    /**
     * 是否有谷歌验证器
     *
     * @param id ID
     * @return true/false
     */
    public boolean hasGoogleOtp(String id) {
        PayMerchant sysUser = getById(id);
        return sysUser.getHasTpop();
    }

    /**
     * 帐号是否存在
     *
     * @param username .
     * @return .
     */
    public Result<Boolean> hashUsername(String username) {
        return Result.OK(loadUserByUsername(username).isPresent());
    }

    /**
     * 根据ID获取详情
     *
     * @param id .
     * @return .
     */
    public Result<PayMerchantVO> detail(String id) {
        PayMerchant user = getById(id);
        PayMerchantVO res = BeanUtil.toBean(user, PayMerchantVO.class);
        return Result.OK(res);
    }


    /**
     * 更新用户
     *
     * @param id .
     * @param vo .
     * @return .
     */
    public Result<PayAgentVO> updateById(String id, PayAgentVO vo) {
        PayMerchant user = getById(id);
        if (null == user) {
            return Result.NG("用户不存在");
        }
        BeanUtil.copyProperties(vo, user);
        updateById(user);
        return Result.OK(vo);
    }


    /**
     * 重置密码
     *
     * @param username .
     * @param passwd   .
     * @return .
     */

    public Result<String> resetPassword(@Nonnull String username, @Nonnull String passwd) {
        String newPassword = PasswordUtil.encoder(passwd);
        mapper.resetPassword(username, newPassword);
        return Result.OK();
    }

    /**
     * 修改密码
     *
     * @param username  .
     * @param oldPasswd .
     * @param newPasswd .
     * @return .
     */
    public Result<String> changePasswd(@Nonnull String username, @Nonnull String oldPasswd, @Nonnull String newPasswd) {
        Optional<PayMerchant> sysUser = loadUserByUsername(username);
        if (sysUser.isPresent()) {
            return Result.NG("用户不存在");
        }
        PayMerchant user = sysUser.get();
        if (!PasswordUtil.matches(oldPasswd, user.getPassword())) {
            return Result.NG("原密码错误");
        }
        String newPassword = PasswordUtil.encoder(newPasswd);
        user.setPassword(newPassword);
        mapper.update(user);
        return Result.OK();
    }


    /**
     * 创建谷歌验证器
     *
     * @param id ID
     * @return otp
     */
    public String createGoogleOtp(String id) {
        PayMerchant sysUser = getById(id);
        String secret = OtpAuthUtil.createSecret();
        sysUser.setTpopCode(secret);
        updateById(sysUser);
        return OtpAuthUtil.getOtpQrCodeUrl(sysUser.getUserName(), secret);
    }

    /**
     * 校验谷歌验证器
     *
     * @param id   ID
     * @param code 验证码
     * @return true/false
     */
    public boolean checkGoogleOtp(String id, String code) {
        PayMerchant sysUser = getById(id);
        String secret = sysUser.getTpopCode();
        if (Boolean.FALSE.equals(sysUser.getHasTpop())) {
            log.warn("用户未设置谷歌验证器");
            return false;
        }
        return OtpAuthUtil.validateCode(secret, code);
    }

    @Override
    public Result<String> bindGoogleOtp(BindGoogleOtpRequest request) {
        UserInfo userInfo = SecurityUtil.getCurrentUser();
        PayMerchant sysUser = getById(userInfo.getUserid());
        String secret = sysUser.getTpopCode();
        var b = OtpAuthUtil.validateCode(secret, request.getCode());
        if (!b) {
            return Result.NG("验证码错误");
        }
        sysUser.setHasTpop(true);
        updateById(sysUser);
        return Result.OK();
    }

}