package pay.platform.core.security.token;

import cn.bzvs.json.JsonUtils;
import jakarta.annotation.Nonnull;
import jakarta.servlet.http.HttpServletRequest;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import pay.platform.api.system.cache.CacheUtil;
import pay.platform.api.system.cache.KeyValue;
import pay.platform.api.system.cache.PlatformCache;
import pay.platform.core.security.config.SecurityProperties;
import pay.platform.core.security.model.OnlineUser;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.util.JwtUtil;

import java.util.Optional;

/**
 * 创建 jwt，并且存储与redis
 */
@Component
@RequiredArgsConstructor
public class JwtTokenRedisCacheProvider implements TokenProvider, CacheUtil {

    private final SecurityProperties properties;

    private final PlatformCache cache;

    @Override
    public String createToken(@Nonnull UserInfo userInfo) {
        return createToken(userInfo, null);
    }

    @Override
    public String createToken(UserInfo userInfo, HttpServletRequest request) {
        String _jwtToken = JwtUtil.sign(userInfo.getUsername(), properties.getJwtSecret());
        OnlineUser onlineUser = OnlineUser.convert(userInfo);
        String json = JsonUtils.toJsonString(onlineUser);
        String key = getCacheKey(CacheKeyValue.online_user_cache, _jwtToken);
        cache.set(key, json, properties.getTokenValidity());
        return _jwtToken;
    }

    @Override
    public Optional<OnlineUser> getOnlineFromToken(@Nonnull String token) {
        String cacheKey = getCacheKey(CacheKeyValue.online_user_cache, token);
        Optional<String> optional = cache.getStr(cacheKey);
        return optional.flatMap(json -> Optional.ofNullable(JsonUtils.stringToBean(json, OnlineUser.class)));
    }

    @Override
    public boolean checkRenewal(@Nonnull String token) {
        if (!properties.isDelayToken()) {
            return false;
        }
        // 获取剩余时间 单位/秒
        String key = getCacheKey(CacheKeyValue.online_user_cache, token);
        long expire = cache.getExpire(key);
        // 是否小于30
        if (expire < properties.getTokenDetect() && expire != 0) {
            // 重新设置过期时间
            cache.expire(key, properties.getTokenValidity());
        }
        return true;
    }

    @Override
    public boolean removeToken(String token) {
        String cacheKey = getCacheKey(CacheKeyValue.online_user_cache, token);
        return cache.del(cacheKey);
    }

    @Getter
    public enum CacheKeyValue implements KeyValue {
        online_user_cache("token", "在线用户"),
        ;
        private final String prefix;
        private final String name;

        CacheKeyValue(String prefix, String name) {
            this.prefix = prefix;
            this.name = name;
        }
    }
}
