package pay.platform.api.controller;

import com.alibaba.fastjson2.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipaySignature;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pay.platform.alipay.enums.AliPayApi;
import pay.platform.api.model.response.OrderResponse;
import pay.platform.api.servcie.*;

import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/api/")
public class CallbackController {
    @Autowired
    private PayPlatformSettingService platformSettingService;
    @Autowired
    private PayOrderService payOrderService;
    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;
    @Autowired
    private PayDailyTotalService payDailyTotalService;
    @Autowired
    private PayChannelService payChannelService;


    /**
     * 支付宝同步回调
     *
     * @param request 请求
     * @param appId   app_id
     * @return 返回结果
     */
    @RequestMapping(value = "/return_url/{appId}")
    @ResponseBody
    public String returnUrl(HttpServletRequest request, @PathVariable String appId) {
        try {
            // 获取支付宝GET过来反馈信息
            Map<String, String> map = AliPayApi.toMap(request);
            for (Map.Entry<String, String> entry : map.entrySet()) {
                log.info("参数：{} - 值： {}", entry.getKey(), entry.getValue());
            }
            var payMerchantAppKey = payMerchantAppKeyService.getByAppId(appId);
            var outTradeNo = map.get("out_trade_no");

            var payOrder = payOrderService.queryOrderByPlatformOrder(payMerchantAppKey.getMerchantId(), outTradeNo);
            if (payOrder == null) {
                log.error("订单不存在：{}", JSONObject.toJSONString(map));
                return "failure";
            }
            var channel = payChannelService.getById(payOrder.getChannelId());
            String publicKey = channel.getPublicKey();
            if (channel.getType() == 1) {
                var payPlatformSetting = platformSettingService.list().get(0);
                // 服务商 - 公钥
                publicKey = payPlatformSetting.getPublicKey();
            }

            boolean verifyResult = AlipaySignature.rsaCheckV1(map, publicKey, "UTF-8", "RSA2");

            if (verifyResult) {
                // TODO 请在这里加上商户的业务逻辑程序代码
                log.info("return_url 验证成功");
//                var payMerchantAppKey = payMerchantAppKeyService.getByAppId(appId);
//                var outTradeNo = map.get("out_trade_no");
                var totalAmount = map.get("total_amount");
                var tradeNo = map.get("trade_no");
//                var payOrder = payOrderService.queryOrderByPlatformOrder(payMerchantAppKey.getMerchantId(), outTradeNo);
//                if (payOrder == null) {
//                    log.error("订单不存在：{}", JSONObject.toJSONString(map));
//                    return "failure";
//                }
                payOrder.setThirdOrderNo(tradeNo);
                payOrder.setStatus(1);
                payOrder.setCallbackInfo(JSONObject.toJSONString(map));
                if (!payOrder.getAmount().toString().equals(totalAmount)) {
                    log.error("金额不匹配：{}", JSONObject.toJSONString(map));
                    payOrderService.save(payOrder);
                    return "failure";
                }

                payOrderService.updateById(payOrder);
                try {
                    //回调商户
                    payOrderService.notifyMerchant(payOrder, payMerchantAppKey);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
                try {
                    //统计当天支付金额
                    payDailyTotalService.addDailyTotal(payOrder);
//                    payDailyTotalService.addWeekTotal(payOrder);

                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }

                //回调下级
                return "success";
            } else {
                log.warn("return_url 验证失败");
                return "failure";
            }
        } catch (AlipayApiException e) {
            log.error("支付宝同步回调异常", e);
            return "failure";
        }
    }

    /**
     * 支付宝同步回调
     *
     * @param request 请求
     * @param appId   app_id
     * @return 返回结果
     */
    @RequestMapping(value = "/return")
    @ResponseBody
    public String returnUrl(@RequestBody OrderResponse orderResponse) {
        log.info("订单信息：{}", JSONObject.toJSONString(orderResponse));
        //订单信息：{"amount":0.01,"currentTime":1716808777567,"orderNo":"1","sign":"17fb4a483ac69c7bd3fd224c04b9d75b","status":1}
        return "success";

    }
}
